#include <menu.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <ncurses.h>
#include <curses.h>
#include <time.h>
#include <unistd.h>
#include <regex.h>
#include <stdlib.h>
#include "defs.h"


void init_curses(void) {

    initscr();  
    getmaxyx(stdscr,max_y,max_x);

    printf("\033[?25l");    /* hide the cursor in term's (and console)... replaces curs_set() */
    printf("\033[?30l");    /* hide scrollbar */
    
    cbreak();
    noecho();
    keypad(stdscr, TRUE);
    
}

void kill_curses(void) {

    printf("\033[?25h");    /* make the cursor visible again */
    endwin();

}


void print_time(int maxx) {

    time_t         t;
    char		*timestr;

    t = time(NULL); 
    timestr = ctime(&t);

    timestr[16]='\0';   /* chop year */
    if (has_colors() && conf.color) {
        attron(COLOR_PAIR(4));  
    }
    mvprintw(conf.border + conf.topspace, maxx-18, "%s", timestr); 
}

void print_channels(void) {

    int i,j=0;

    if (has_colors() == TRUE && conf.color == 1) {
        attron(COLOR_PAIR(1));  
    }
    for (i=0; i<16; i++) {
        if (kanaler[i].used != 1) break;
        mvprintw(conf.topspace+conf.clock+conf.border+j,1+conf.border,"%s",kanaler[i].name);
        j++;
    }
}

void actual_print(int chann, int i) {
    
    char tempstr[32];
    memset(tempstr, ' ', 32);

    if (kanaler[chann].program[i][0] == '\0') return;

    if (has_colors() && conf.color)
        attron(COLOR_PAIR(2));
    mvprintw(conf.topspace + conf.clock + conf.border + chann,14+conf.border,"%.2d:%.2d ",(kanaler[chann].starttid[i] - kanaler[chann].starttid[i]%60)/60, kanaler[chann].starttid[i]%60, kanaler[chann].program[i]);

    if (has_colors() && conf.color) 
        attron(COLOR_PAIR(5));       
    if (kanaler[chann].program[i+1][0])
        mvprintw(conf.topspace+conf.clock + conf.border + chann,53+conf.border,"%.2d:%.2d ",(kanaler[chann].starttid[i+1] - kanaler[chann].starttid[i+1]%60)/60, kanaler[chann].starttid[i+1]%60, kanaler[chann].program[i+1]);
    else
        mvprintw(conf.topspace+conf.clock + conf.border + chann,53+conf.border,"      ");
    if (kanaler[chann].watch[i] == 1)
        attron(A_BOLD);  
    if (has_colors() == TRUE && conf.color == 1) 
        attron(COLOR_PAIR(3));

    strncpy(tempstr, kanaler[chann].program[i], strlen(kanaler[chann].program[i])); /* padding */
    tempstr[31] = '\0';

    mvprintw(conf.topspace+conf.clock+conf.border+chann,20+conf.border,"%s", tempstr);
    attroff(A_BOLD); 

    if (kanaler[chann].watch[i+1] == 1) 
        attron(A_BOLD);

    if (has_colors() == TRUE && conf.color == 1)
        attron(COLOR_PAIR(6));       

    memset(tempstr, ' ', 32);
    if (kanaler[chann].program[i+1][0]) 
        strncpy(tempstr, kanaler[chann].program[i+1], strlen(kanaler[chann].program[i+1])); /* padding */
    tempstr[31] = '\0';

    if (max_x-(59+conf.border) < 32) 
        tempstr[max_x-(59+conf.border)-1] = '\0';
    mvprintw(conf.topspace+conf.clock+conf.border+chann,59+conf.border,"%s", tempstr);  
    attroff(A_BOLD);
}

void special_print(int chann) {

    char tempstr[32];
    memset(tempstr, ' ', 32);

    if (has_colors() == TRUE && conf.color == 1) {
        attron(COLOR_PAIR(5));
    }
    mvprintw(conf.border+conf.topspace+conf.clock+chann,53+conf.border,"%.2d:%.2d ",(kanaler[chann].starttid[0] - kanaler[chann].starttid[0]%60)/60, kanaler[chann].starttid[0]%60, kanaler[chann].program[0]);

    if (kanaler[chann].watch[0] == 1) {
        attron(A_BOLD);
    }
    if (has_colors() == TRUE && conf.color == 1) {
        attron(COLOR_PAIR(6));
    }
    mvprintw(conf.border+conf.topspace+conf.clock+chann,14+conf.border,"                                     "); /* clear the current field */


    strncpy(tempstr,kanaler[chann].program[0], strlen(kanaler[chann].program[0]));
    tempstr[31] = '\0';
    if (max_x-(59+conf.border) < 32) {
        tempstr[max_x-(59+conf.border)-1] = '\0';
    }
    mvprintw(conf.border+conf.topspace+conf.clock+chann,59+conf.border,"%s", tempstr);
    attroff(A_BOLD);
}


void print_curr_prgms(int chann) {

    time_t         t		= time(NULL);
    struct tm      *tid		= localtime(&t);
    int            i, mn_mins	= ((((tid->tm_hour)%24)*60)+ tid->tm_min);  /* mins since midnight */

    for (i=0;i<MAXCHAN && kanaler[chann].program[i][0] != '\0' ;i++) { 
        if (kanaler[chann].starttid[i] <= mn_mins && kanaler[chann].starttid[i+1] > mn_mins) {
            actual_print(chann, i);
            return;
        }
    }
    /* not caught by simple logic */

    for (i=0;i<MAXCHAN && kanaler[chann].program[i][0] != '\0' ;i++) {
        if (kanaler[chann].starttid[i] <= mn_mins && kanaler[chann].starttid[i+1] < kanaler[chann].starttid[i]) {
            if (kanaler[chann].starttid[0] > mn_mins && mn_mins >= 360) {
                special_print(chann);
                return;
            }
            actual_print(chann, i);
            return;
        }
        else if (kanaler[chann].starttid[i] > mn_mins && kanaler[chann].starttid[i+1] > mn_mins &&
                kanaler[chann].starttid[i] > kanaler[chann].starttid[i+1]) {
            actual_print(chann, i);
            return;     		
        }
    }
    /* are we at the last program, after the last program of before the first? */  

    if (mn_mins < kanaler[chann].starttid[i-1])
        actual_print(chann, 0);
    else
        actual_print(chann, i-1);
}

void print_all() {

    int i;

    if (conf.clock)
        print_time(max_x);
    print_channels();

    for (i=0;i<16;i++) {
        if (kanaler[i].used != 1) break;
        print_curr_prgms(i);    
    }
    refresh();
    if (win_watch) {
        touchwin(win_watch);
        wrefresh(win_watch);
    }
    if (win_find) {
        touchwin(win_find);
        wrefresh(win_find);
    }
    if (win_list) {
        touchwin(win_list);
        wrefresh(win_list);
    }
    if (win_tw) {
        touchwin(win_tw);
        wrefresh(win_tw);
    }
}

WINDOW *print_reload_win(void) {

    WINDOW *reloadwin;
    reloadwin = newwin(3, 20, (max_y/2)-1,(max_x/2)-10);

    if (has_colors() == TRUE && conf.color == 1) {
        wattron(reloadwin,COLOR_PAIR(7));
    }
    box(reloadwin,0,0);

    if (has_colors() == TRUE && conf.color == 1) {
        wattron(reloadwin,COLOR_PAIR(8));
    }

    mvwprintw(reloadwin,1,2,"Reloading...");

    wrefresh(reloadwin);
    return reloadwin;
}

WINDOW *print_watch_win(void) {

    int i,j=0,k;
    WINDOW *watchwin;
    MENU    *menu_list;
    ITEM    **item_list;
    int     c;
    char    tmpwatch[32];
    char    cont = 1; 
    
    memset(&tmpwatch, 0, sizeof(tmpwatch));
    
    for (i=0; i < MAXWATCH; i++) {
        if(conf.watch[i][0] == '\0') break;
        j++;
    }
    if (j == 0) {
        /* no watches set */ 
        watchwin = error_win("No watches set...");
        return watchwin;
    }
    
    item_list = (ITEM **) calloc(j+1, sizeof(ITEM *));
    
    for (k=0; k < j; k++) {
        item_list[k] = new_item(" ",conf.watch[k]);
    }
    
    item_list[j] = (ITEM *) NULL;
    menu_list = new_menu((ITEM **) item_list);
    
    if ((watchwin = newwin(max_y-2-conf.border-conf.topspace-conf.clock, 36, conf.topspace + conf.border + conf.clock + 1, (max_x/2)-18)) != NULL) {

        keypad(watchwin, TRUE);

        if (has_colors() && conf.color) { 
            set_menu_back(menu_list, COLOR_PAIR(8));
            set_menu_fore(menu_list, COLOR_PAIR(8) | A_BOLD);
        }

        set_menu_win(menu_list, watchwin);
        set_menu_sub(menu_list, derwin(watchwin, max_y-4-conf.border-conf.topspace-conf.clock, 34, 1, 2));
        set_menu_format(menu_list, max_y-4-conf.border-conf.topspace-conf.clock, 1);
        set_menu_mark(menu_list, "*");

        if (has_colors() && conf.color) {
            wattron(watchwin,COLOR_PAIR(7));
        }

        box(watchwin,0,0);

        post_menu(menu_list);
        wrefresh(watchwin);

        while ((c = wgetch(watchwin)) != 27 && cont == 1) {
            switch (c) {
                case KEY_DOWN:    /* down */
                    menu_driver(menu_list, REQ_DOWN_ITEM);
                    break;
                case KEY_UP:    /* up */
                    menu_driver(menu_list, REQ_UP_ITEM);
                    break;
                case KEY_PPAGE:   /* page up */
                    menu_driver(menu_list, REQ_SCR_UPAGE);
                    break;
                case KEY_NPAGE:   /* page down */
                    menu_driver(menu_list, REQ_SCR_DPAGE);
                    break;
                case 100: /* d */
                    /* FIXME: uuuuuuuuugly * 10^99! */
                    strncpy(tmpwatch, item_description(current_item(menu_list)), 32);
                    unpost_menu(menu_list);
                    free_menu(menu_list);
                    watchwin = kill_win(watchwin);
                    for (k=0; k < j; k++)
                        free_item(item_list[k]);
                    delete_watch_from_cfg(tmpwatch);
                    refresh();
                    print_border();
                    print_all();
                    cont = 0;
                    break;
            }
        }
        if (cont) {
            unpost_menu(menu_list);
            free_menu(menu_list);
            watchwin = kill_win(watchwin);
        }
    }
    
    if (cont) {
        for (k=0; k < j; k++)
            free_item(item_list[j]);
    } else {
        watchwin = print_watch_win();
    }

    return watchwin;
}


WINDOW *kill_win(WINDOW *killwin) {

    delwin(killwin);
    clear();
    return NULL;
}
void check_resize() {

    int x, y;

    getmaxyx(stdscr,y,x);
    mvprintw(max_y-conf.border-1,max_x-conf.border-1,""); /* don't ask... */
    if ((x != max_x) || (y != max_y)) {
        getmaxyx(stdscr,max_y,max_x);
        clear();
        printf("\033[?25l"); /* aterm seems to turn the cursor back on when resizeing */
        print_border();
        print_all();
    }

}


void print_border(void) {

    if (conf.border) {
        if (has_colors() == TRUE && conf.color == 1) {
            attron(COLOR_PAIR(7));
        }
        box(stdscr, 0, 0);   
    }
    getmaxyx(stdscr,max_y,max_x);
    refresh(); 
}

void init_colors() {

    if (has_colors() == TRUE && conf.color == 1) {  /* colors if avail and wanted */
        start_color();
        use_default_colors();
        
        init_pair(1, COLOR_RED, -1);  /* channel color */
        init_pair(2, COLOR_CYAN, -1);  /* time color */
        init_pair(3, COLOR_GREEN, -1);  /* program color */
        init_pair(4, COLOR_RED, -1); /* clock color */
        init_pair(5, COLOR_CYAN, -1); /* next prgtime color */        
        init_pair(6, COLOR_BLUE, -1); /* next prg color */
        init_pair(7, COLOR_MAGENTA, -1); /* boxborder color   */
        init_pair(8, COLOR_WHITE, -1);  /* textcolor (reloading, watch) */
    }
}

WINDOW *print_todays_watch(void) {

    int i=0,k,l;
    int j = count_watches();
    WINDOW *watchwin;

    MENU    *menu_list;
    ITEM    **item_list;
    int     c;
    struct prgtable twatch[j];
    struct prgtable *tp[j];
    char   str[j][32];
    char   time[j][20];
/*
    for (k = 0; k < 16 && kanaler[k].used == 1; k++) {
        for (l = 0; l < MAXCHAN && kanaler[k].program[l][0] != ' '; l++) {  
            if (kanaler[k].watch[l] == 1) {
                j++;
            }
        }
    }
*/    
    if (j == 0) {
        /* no watches today */ 
        watchwin = error_win("No watches today...");
        return watchwin;
    }
/*    
    for (k = 0; k < j; k++)
        memset(&twatch[k], 0, sizeof(twatch[k]));
*/    
    memset(&twatch,0, sizeof(twatch));
    
    for (k = 0; k < 16 && kanaler[k].used == 1; k++) {
        for (l = 0; l < MAXCHAN && kanaler[k].program[l][0] != ' '; l++) {  
            if (kanaler[k].watch[l] == 1) {
                memcpy(twatch[i].channel, kanaler[k].name, sizeof(twatch[i].channel));
                memcpy(twatch[i].program, kanaler[k].program[l], sizeof(twatch[i].program));
                twatch[i].starttid = kanaler[k].starttid[l];
                tp[i] = &twatch[i];
                i++;
            }
        }
    }
    
    qsort(&twatch[0], j, sizeof(struct prgtable), compare_func);
    
    i=0;
    
    for (k = 0; k < j; k++) 
        if (tp[k]->starttid > 360) {
            sprintf(time[i],"%-12.12s %.2d:%.2d ",tp[k]->channel,(tp[k]->starttid - tp[k]->starttid%60)/60, tp[k]->starttid%60);
            sprintf(str[i],"%s", tp[k]->program);
            i++;
        }
    
    for (k = 0; k < j; k++) 
        if (tp[k]->starttid <= 360) {
            sprintf(time[i],"%-12.12s %.2d:%.2d ",tp[k]->channel,(tp[k]->starttid - tp[k]->starttid%60)/60, tp[k]->starttid%60);
            sprintf(str[i],"%s", tp[k]->program);
            i++;
        }
     
    item_list = (ITEM **) calloc(i+1, sizeof(ITEM *));
    
    for (k=0; k < i; k++) {
        item_list[k] = new_item(time[k], str[k]);
    }
    item_list[i] = (ITEM *) NULL;
    menu_list = new_menu((ITEM **) item_list);
    
    
    if ((watchwin = newwin(max_y-2-conf.border-conf.topspace-conf.clock, 56, conf.topspace + conf.border + conf.clock + 1, (max_x/2)-28)) != NULL) {

        keypad(watchwin, TRUE);

        if (has_colors() && conf.color) { 
            set_menu_back(menu_list, COLOR_PAIR(8));
            set_menu_fore(menu_list, COLOR_PAIR(8) | A_BOLD);
        }

        set_menu_win(menu_list, watchwin);
        set_menu_sub(menu_list, derwin(watchwin, max_y-4-conf.border-conf.topspace-conf.clock, 54, 1, 2));
        set_menu_format(menu_list, max_y-4-conf.border-conf.topspace-conf.clock, 1);
        set_menu_mark(menu_list, "*");

        if (has_colors() && conf.color) {
            wattron(watchwin,COLOR_PAIR(7));
        }

        box(watchwin,0,0);

        post_menu(menu_list);
        wrefresh(watchwin);

        while ((c = wgetch(watchwin)) != 27) {
            switch (c) {
                case KEY_DOWN:    /* down */
                    menu_driver(menu_list, REQ_DOWN_ITEM);
                    break;
                case KEY_UP:    /* up */
                    menu_driver(menu_list, REQ_UP_ITEM);
                    break;
                case KEY_PPAGE:   /* page up */
                    menu_driver(menu_list, REQ_SCR_UPAGE);
                    break;
                case KEY_NPAGE:   /* page down */
                    menu_driver(menu_list, REQ_SCR_DPAGE);
                    break;
            }
        }

        unpost_menu(menu_list);
        free_menu(menu_list);
        watchwin = kill_win(watchwin);
    }
    
    for (k=0; k < i; k++)
        free_item(item_list[k]);
    
    
    return NULL;
}

int compare_func(const void *a, const void *b) {

    struct prgtable *x,*y;

    y = (struct prgtable *) b;
    x = (struct prgtable *) a;

    if (x->starttid < y->starttid)
        return -1;
    else if (x->starttid == y->starttid)
        return 0;
    else
        return 1;
}

void edit_tvconf(void) {

    char   conf[512];
    printf("\033[?25h");    /* make cursor visible again */
    def_prog_mode();
    
    memset(&conf, 0, sizeof(conf));
    snprintf(conf, sizeof(conf)-1, "%s %s/.tvconfig", getenv("EDITOR"), getenv("HOME"));
    system(conf); 
    
    reset_prog_mode();
    refresh();
    printf("\033[?25l");

}

WINDOW *find_program(void) {

    WINDOW  *findwin;
    MENU    *menu_list;
    ITEM    **item_list;
    
    char   findstr[32];
    int    i,j,k,hits = 0;
    char   msg[50];
    int    c;
    regex_t rexp;
    
    struct prgtable hitrecord[conf.maxhits];
    struct prgtable *hp[conf.maxhits];
    char   str[conf.maxhits][32];
    char   time[conf.maxhits][20];

    
    memset(&str, 0, sizeof(str));
    memset(&time, 0, sizeof(time));
    memset(&findstr[0], 0, sizeof(findstr));

    for (k = 0; k < conf.maxhits; k++)
        memset(&hitrecord[k], 0, sizeof(hitrecord[k]));

    if ((findwin = newwin(3, 44, (max_y/2)-1,(max_x/2)-17)) != NULL ) {
        if (has_colors() == TRUE && conf.color == 1) {
            wattron(findwin,COLOR_PAIR(7));
        }
        box(findwin,0,0);

        if (has_colors() == TRUE && conf.color == 1) {
            wattron(findwin,COLOR_PAIR(8));
        }
        echo();
        mvwprintw(findwin, 1, 2,"Find: ");
        printf("\033[?25h");    /* make the cursor visible again */
        if ((mvwgetnstr(findwin, 1, 8, &findstr[0], 31)) == ERR) {
            findwin = kill_win(findwin);
            return NULL;
        }
        noecho();
        printf("\033[?25l");    /* hide cursor */


        regcomp(&rexp, findstr, REG_EXTENDED | REG_ICASE);

        for (i = 0; (i < 16 && kanaler[i].used == 1); i++)
            for (j = 0; (j < MAXCHAN && kanaler[i].program[j][0] != ' '); j++)
                if ((regexec(&rexp, kanaler[i].program[j], 0, NULL, 0)) == 0) { 
                    if (hits >= conf.maxhits)
                        break;
                    memcpy(hitrecord[hits].channel, kanaler[i].name, sizeof(hitrecord[hits].channel));
                    memcpy(hitrecord[hits].program, kanaler[i].program[j], sizeof(hitrecord[hits].program));
                    hitrecord[hits].starttid = kanaler[i].starttid[j];
                    hp[hits] = &hitrecord[hits];
                    hits++;
                }

        regfree(&rexp);


        findwin = kill_win(findwin);
        print_border();
        print_all();

        if (hits == 0) {  /* no hits */
            sprintf(msg,"No hits on: %s",findstr);
            findwin = error_win(msg);
            return findwin;
        }
        
        qsort(&hitrecord[0], hits, sizeof(struct prgtable), compare_func);

        i=0;

        for (k = 0; k < hits; k++) 
            if (hp[k]->starttid > 360) {
                sprintf(time[i],"%-12.12s %.2d:%.2d ",hp[k]->channel,(hp[k]->starttid - hp[k]->starttid%60)/60, hp[k]->starttid%60);
                sprintf(str[i],"%s", hp[k]->program);
                i++;
            }
        for (k = 0; k < hits; k++) 
            if (hp[k]->starttid <= 360) {
                sprintf(time[i],"%-12.12s %.2d:%.2d ",hp[k]->channel,(hp[k]->starttid - hp[k]->starttid%60)/60, hp[k]->starttid%60);
                sprintf(str[i],"%s", hp[k]->program);
                i++;
            }

        item_list = (ITEM **) calloc(i+1, sizeof(ITEM *));

        for (k=0; k < i; k++) {
            item_list[k] = new_item(time[k], str[k]);
        }
        item_list[i] = (ITEM *) NULL;
        menu_list = new_menu((ITEM **) item_list);


        if ((findwin = newwin(max_y-2-conf.border-conf.topspace-conf.clock, 56, conf.topspace + conf.border + conf.clock + 1, (max_x/2)-28)) != NULL) {

            keypad(findwin, TRUE);

            if (has_colors() && conf.color) { 
                set_menu_back(menu_list, COLOR_PAIR(8));
                set_menu_fore(menu_list, COLOR_PAIR(8) | A_BOLD);
            }

            set_menu_win(menu_list, findwin);
            set_menu_sub(menu_list, derwin(findwin, max_y-4-conf.border-conf.topspace-conf.clock, 54, 1, 2));
            set_menu_format(menu_list, max_y-4-conf.border-conf.topspace-conf.clock, 1);
            set_menu_mark(menu_list, "*");

            if (has_colors() && conf.color) {
                wattron(findwin,COLOR_PAIR(7));
            }

            box(findwin,0,0);

            post_menu(menu_list);
            wrefresh(findwin);

            while ((c = wgetch(findwin)) != 27) {
                switch (c) {
                    case KEY_DOWN:    /* down */
                        menu_driver(menu_list, REQ_DOWN_ITEM);
                        break;
                    case KEY_UP:    /* up */
                        menu_driver(menu_list, REQ_UP_ITEM);
                        break;
                    case KEY_PPAGE:   /* page up */
                        menu_driver(menu_list, REQ_SCR_UPAGE);
                        break;
                    case KEY_NPAGE:   /* page down */
                        menu_driver(menu_list, REQ_SCR_DPAGE);
                        break;
                    case 97:  /* a */
                        add_watch(item_description(current_item(menu_list)));
                        print_border();
                        print_all();
                        touchwin(findwin);
                        wrefresh(findwin);
                        break;
                }
            }
            unpost_menu(menu_list);
            free_menu(menu_list);
            findwin = kill_win(findwin);
        }

        for (k=0; k < i; k++)
            free_item(item_list[k]);

        
    }
    return NULL;

}

WINDOW *list_channel(void) {

    int i;
    int numchans = 0;
    int c;
    MENU *menu_chan;
    WINDOW *menu_win; 
    ITEM **item_chan;
    char cont = 1;
    
    for (i=0; i < 16; i++)
        if (kanaler[i].used == 1)
            numchans++;

    item_chan = (ITEM **) calloc(numchans+1, sizeof(ITEM *));
    
    for (i=0; i < numchans; i++)
        item_chan[i] = new_item(kanaler[i].name, NULL);
    
    item_chan[numchans] = (ITEM *) NULL;
    menu_chan = new_menu((ITEM **) item_chan);
    
    if ((menu_win = newwin(numchans, 13, conf.topspace + conf.border + conf.clock, conf.border)) != NULL) {

        keypad(menu_win, TRUE);

        if (has_colors() && conf.color) { 
            set_menu_back(menu_chan, COLOR_PAIR(1));
            set_menu_fore(menu_chan, COLOR_PAIR(1) | A_BOLD);
        }

        set_menu_win(menu_chan, menu_win);
        set_menu_sub(menu_chan, derwin(menu_win, numchans, 13, 0, 0));
        set_menu_mark(menu_chan, "*");

        post_menu(menu_chan);
        wrefresh(menu_win);
        
        
        while ((c = wgetch(menu_win)) != 27 && cont == 1) {
            switch (c) {
                case KEY_DOWN:    /* down */
                    menu_driver(menu_chan, REQ_DOWN_ITEM);
                    break;
                case KEY_UP:    /* up */
                    menu_driver(menu_chan, REQ_UP_ITEM);
                    break;
                case KEY_PPAGE:   /* page up */
                    menu_driver(menu_chan, REQ_SCR_UPAGE);
                    break;
                case KEY_NPAGE:   /* page down */
                    menu_driver(menu_chan, REQ_SCR_DPAGE);
                    break;
                case 10:
                    for (i=0; i < numchans; i++) { 
                        if ((strcmp(kanaler[i].name, item_name(current_item(menu_chan)))) == 0) {
                            show_channel_listing(i);
                        }
                    }
                    cont = 0;
                    break;
            }
        }
                    
        unpost_menu(menu_chan);
        free_menu(menu_chan);
        menu_win = kill_win(menu_win);
    }
    else {
        menu_win = error_win("Can't fit menu on screen!");
    }
    
    for (i=0; i < numchans; i++)
        free_item(item_chan[i]);

    return menu_win;
}
            

WINDOW *error_win(char *msg) {

    WINDOW *errwin;
    int msglen;

    msglen = strlen(msg);
    
    if ((errwin = newwin(3, msglen+4, (max_y/2)-1,(max_x/2)-((msglen+4)/2))) != NULL) {
        if (has_colors() && conf.color) {
            wattron(errwin,COLOR_PAIR(7));
        }
        box(errwin,0,0);

        if (has_colors() && conf.color) {
            wattron(errwin,COLOR_PAIR(8));
        }
        mvwprintw(errwin,1,2,"%s",msg);
        wrefresh(errwin);
        return errwin;

    }
    
    beep(); /* hopefully gives a hint that there is something wrong.. :) */
    return NULL;
}

void show_channel_listing(int channel) {

    WINDOW  *list_win;
    MENU    *menu_list;
    ITEM    **item_list;
    int     i, nrprogs = 0;
    char    str[MAXCHAN][35];
    char    time[MAXCHAN][7];
    int     c;

    memset(&str,0,sizeof(str));
    memset(&time, 0, sizeof(time));
    
    for (i=0;i<MAXCHAN && kanaler[channel].program[i][0] != '\0' ;i++) {
        nrprogs++;
        sprintf(time[i], "%.2d:%.2d ",(kanaler[channel].starttid[i] - kanaler[channel].starttid[i]%60)/60, kanaler[channel].starttid[i]%60);
        sprintf(str[i], "%s", kanaler[channel].program[i]);
        
    }
    
    item_list = (ITEM **) calloc(nrprogs+1, sizeof(ITEM *));
    
    for (i=0; i < nrprogs; i++) {
        item_list[i] = new_item(time[i], str[i]);
    }
    item_list[nrprogs] = (ITEM *) NULL;
    menu_list = new_menu((ITEM **) item_list);
    
    if ((list_win = newwin(max_y-2-conf.border-conf.topspace-conf.clock, 43, conf.topspace + conf.border + conf.clock + 1, (max_x/2)-21)) != NULL) {
        keypad(list_win, TRUE);

        if (has_colors() && conf.color) { 
            set_menu_back(menu_list, COLOR_PAIR(8));
            set_menu_fore(menu_list, COLOR_PAIR(8) | A_BOLD);
        }

        set_menu_win(menu_list, list_win);
        set_menu_sub(menu_list, derwin(list_win, max_y-4-conf.border-conf.topspace-conf.clock, 41, 1, 2));
        set_menu_format(menu_list, max_y-4-conf.border-conf.topspace-conf.clock, 1);
        set_menu_mark(menu_list, "*");

        if (has_colors() && conf.color) {
            wattron(list_win,COLOR_PAIR(7));
        }

        box(list_win,0,0);

        post_menu(menu_list);
        wrefresh(list_win);

        while ((c = wgetch(list_win)) != 27) {
            switch (c) {
                case KEY_DOWN:    /* down */
                    menu_driver(menu_list, REQ_DOWN_ITEM);
                    break;
                case KEY_UP:    /* up */
                    menu_driver(menu_list, REQ_UP_ITEM);
                    break;
                case KEY_PPAGE:   /* page up */
                    menu_driver(menu_list, REQ_SCR_UPAGE);
                    break;
                case KEY_NPAGE:   /* page down */
                    menu_driver(menu_list, REQ_SCR_DPAGE);
                    break;
                case 97:  /* a */
                    add_watch(item_description(current_item(menu_list)));
                    print_border();
                    print_all();
                    touchwin(list_win);
                    wrefresh(list_win);
                    break;
            }
        }
        clear();
        print_border();
        print_all();
        
        unpost_menu(menu_list);
        free_menu(menu_list);
        list_win = kill_win(list_win);
    }
    
    for (i=0; i < nrprogs; i++)
        free_item(item_list[i]);
    
    return;
}

int count_watches(void) {

    int k, j=0, l;

    for (k = 0; k < 16 && kanaler[k].used == 1; k++) {
        for (l = 0; l < MAXCHAN && kanaler[k].program[l][0] != ' '; l++) {
            if (kanaler[k].watch[l] == 1) {
                j++;
            }
        }
    }

    return j;
}

