#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <unistd.h>
#include <stdlib.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <errno.h>
#include <fcntl.h>
#include <regex.h>
#include <ctype.h>

#include "defs.h"

void initdata(void) { 
    
    memset(&kanaler, 0, sizeof(kanaler)); 
    
    memset(&conf, 0, sizeof(conf));
    conf.color=0;
    conf.border=0;
    conf.clock=0;
    conf.topspace=0;
    conf.maxhits=20;
    conf.forking=0; 
    conf.alarm1=0;       
    conf.alarm2=0;

    win_watch = NULL;
    win_find = NULL;
    win_tw = NULL;
    win_list = NULL;

}

int reload(int bootup) { /* call ONLY with bootup=1 from main() */
    
    FILE *f;
    char buf[512], *p;
    int currwatch=0;
    static int fd;
    

    snprintf(buf, sizeof(buf), "%s/.tvconfig", getenv("HOME"));
    
    if ((f = fopen(buf,"r")) == NULL) { 
        printf("Can't open %s/.tvconfig\n",getenv("HOME"));
        exit(-1); /* cant open config! nothing to do */
    }

    initdata(); /* reset lists */

    while (fgets(buf, sizeof(buf)-1, f) != NULL) {
        parse_data(buf);

        if (strcasecmp(buf, "COLOR") == 0) conf.color=1;
        else if (strcasecmp(buf, "TOPSPACE") == 0) conf.topspace=1;
        else if (strcasecmp(buf, "BORDER") == 0) conf.border=1;
        else if (strcasecmp(buf, "CLOCK") == 0) conf.clock=1;
        
        if ((p = strchr(buf, ' ')) != NULL) { 
            *p++ = '\0';
            if (strcasecmp(buf, "CHANNEL") == 0) filldata(p);
            else if (strcasecmp(buf, "FORK") == 0) { 
                conf.forking = 1; 
                strncpy(conf.tty, p, sizeof(conf.tty)-1);
                conf.tty[sizeof(conf.tty)-1]='\0';

                if(bootup == 1 && fork() != 0) exit(0);
                if(bootup == 1) { close(0); close(1); close(2); }
                else { close(fd); } 
                if ((fd = open(p, O_RDWR)) == -1) exit(-1); /* not much to do.. */
            }  
            else if (strcasecmp(buf, "WATCH") == 0) {
                if (currwatch == MAXWATCH) break;
                strncpy(conf.watch[currwatch], p, 32);
                conf.watch[currwatch++][31] = '\0';
            }
            else if (strcasecmp(buf, "ALARM1") == 0) { 
                conf.alarm1 = (char) atoi(p);
            }   
            else if (strcasecmp(buf, "ALARM2") == 0) { 
                conf.alarm2 = (char) atoi(p);
            }   
            else if (strcasecmp(buf, "EXTALARM1") == 0) { 
                strncpy(conf.extalarm1, p, sizeof(conf.extalarm1));
            }   
            else if (strcasecmp(buf, "EXTALARM2") == 0) { 
                strncpy(conf.extalarm2, p, sizeof(conf.extalarm2));
            }
            else if (strcasecmp(buf, "MAXHITS") == 0) {
                conf.maxhits = (char) atoi(p);   
            }
        }
    }

    
    if (conf.border)
        conf.topspace = 0;
    
    
    if(findfreekanal() == 0) { /* NO CHANNELS?!? */
        exit(0); 
    }  
    fclose(f);

    flag_watches(currwatch);
    
    return 0;
}

int filldata(char *file) {

    struct sockaddr_in	sin;
    int    sfd;
    int    len, freekanal, currprogram = 0, i = 0;
    char   *buf;
    char   *p, *q, *t, *r;
    char   recvbuf[4096];
    char   cpmongo = 0;

    if (file == NULL)
        return(-1);

    if ((sfd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1 ) return(-1);
    sin.sin_family = PF_INET;
    sin.sin_port = htons(80);
    sin.sin_addr.s_addr = inet_addr("212.105.63.152"); /* tvprogram.nu */

    memset(&sin.sin_zero,0,sizeof(sin.sin_zero));

    if (connect(sfd, (struct sockaddr *)&sin, sizeof(sin)) == -1 ) { close(sfd); return(-1); }

    sprintf(recvbuf, "\nGET /cgi-tvprogram/pp1.cgi?%s HTTP/1.1\nHOST: blaha.com\n\nBYE\n\n", file); 
    if (send(sfd,recvbuf,strlen(recvbuf),0) == -1) { printf("error sending for %s\n", file); perror(NULL);close(sfd); return(-1); }
    if ((buf = p = (char *) malloc(100000)) == NULL) { close(sfd); return(-1); }
    memset(buf, 0, 100000);

    while ((len = recv(sfd,recvbuf,sizeof(recvbuf),0)) > 0) {
        if (p-buf+len > 100000) { printf("error reading, too much\n");close(sfd); free(buf); return(-1); } /* heh. someone rooted tvprogram.nu just for me? :P  */
        memcpy(p, recvbuf, len);  
        p += len;
    }

    p = buf;
    strip_cpcr(buf);
    if ((freekanal = findfreekanal()) == -1) { free(buf); close(sfd); return(-1); }
    kanaler[freekanal].used = 1;

    while((p = strstr(p, "<b>")) != NULL ) {
        p += 3;
        q = strstr(p, "</b>");
        *q = '\0';
        parse_data(p);
        
        if (*p == '\0') { p = q+3; continue; }
        if (currprogram == MAXCHAN) break; /* FALL BACK FALL BACK FALL BACK */

        if (i == 0) { 
            if (isdigit(*p)) {
                cpmongo = 1;
                if ((r = strchr(file,'-'))) {
                    *r = '\0';
                    strcpy(kanaler[freekanal].name, file);
                    kanaler[freekanal].name[31] = '\0';
                }
                else {
                    strcpy(kanaler[freekanal].name, "Unknown");
                    kanaler[freekanal].name[31] = '\0';
                }
            }
            else {
                strcpy(kanaler[freekanal].name, p);
                kanaler[freekanal].name[31] = '\0'; /* NULLTERM! */
            }
        }
        else if (i%2 == 0) { /* program name */
            memcpy(kanaler[freekanal].program[currprogram], p, (strlen(p) > 31) ? 31 : strlen(p)-1);
            kanaler[freekanal].program[currprogram][31] = '\0';
            currprogram++;
        }
        else { /* time */
            if ((t = strchr(p, ':')) == NULL && (t = strchr(p, '.')) == NULL) { /* great. why not use ! too. */
                free(buf);
                close(sfd);
                kanaler[freekanal].used = 0;
                return(-1);
            }
            *t++ = '\0'; 
            kanaler[freekanal].starttid[currprogram] = atoi(t) + 60*atoi(p);
        }    

        if (cpmongo == 1) {
            p = buf;
            cpmongo = 0;
            *q = '<';
        }
        else 
            p = q+3;
        
        i++;
    }  

    free(buf);
    close(sfd);
    return(1);
}

void parse_data(char *p) /* p is NULL terminated */
{
    char *q;
    
    if (*(p+strlen(p)-1) == '\n' || *(p+strlen(p)-1) == '\r') {
        *(p+strlen(p)-1) = '\0';
        parse_data(p);
    }     
   
    /* fix ' and ` */
    
    while ((q = strchr(p,0x92)) != NULL)
        *q = '\'';

    while ((q = strchr(p,0x91)) != NULL)
        *q = '`';

}

void strip_cpcr(char *buf) { /* buf is NULL terminated */
                                
    char *p, *q;

    if ((p = strchr(buf, '\r')) == NULL || (q = strchr(p+1, '\r')) == NULL) return;  
    if (*(q+1) == '\n') q++;

    strcpy(p,q+1);
    strip_cpcr(p);
}

int findfreekanal(void) {
    int i;
    for (i=0;i<16;i++)
        if (kanaler[i].used == 0) return i;
    return(-1);
}

void delete_watch_from_cfg(const char *str) {


    FILE *f, *f2;
    
    int  currwatch = 0;
    char cfgfile[512];
    char tmpfile[512];
    char buf[512];
    char *p;
    
    memset(&conf.watch, 0, sizeof(conf.watch));
    
    snprintf(cfgfile, sizeof(cfgfile), "%s/.tvconfig", getenv("HOME"));
    snprintf(tmpfile, sizeof(tmpfile), "%s/.tvconfig.tmp", getenv("HOME"));

    if ((f = fopen(cfgfile,"r")) == NULL) {
        beep();
        return;
    }

    if ((f2 = fopen(tmpfile, "w")) == NULL) {
        beep();
        return;
    }

    while ((fgets(buf, sizeof(buf)-1, f)) != NULL) {
        if ((strncasecmp(buf, "WATCH", 5)) == 0) {
            if ((p = strchr(buf, ' ')) != NULL) {
                p++;
                if ((strncmp(str, p, strlen(p)-1)) == 0)
                    continue;
                strncpy(conf.watch[currwatch], p, strlen(p)-1);
                conf.watch[currwatch++][31] = '\0';
            }
        }
        fputs(buf,f2);
    }
    
    fclose(f);
    fclose(f2);
     
    rename(tmpfile, cfgfile);
    
    unflag_watch(str);
    
    return;

}

void flag_watches(int num) {

    int i,j,k;
    
    regex_t rexp[num+1];

    for (k = 0; k < num; k++) 
        regcomp(&rexp[k], conf.watch[k], REG_EXTENDED | REG_ICASE);

    for(k = 0; k < num; k++)
        for(i = 0; (i < 16 && kanaler[i].used == 1); i++)
            for(j = 0; (j < MAXCHAN && kanaler[i].program[j][0] != ' '); j++)
                if ((regexec(&rexp[k], kanaler[i].program[j], 0, NULL, 0)) == 0)  
                    kanaler[i].watch[j] = 1;

    for (k = 0; k < num; k++)
        regfree(&rexp[k]);
    
}

void unflag_watch(const char *str) {

    int i,j;
    
    regex_t rexp;

    regcomp(&rexp, str, REG_EXTENDED | REG_ICASE);

    for(i = 0; (i < 16 && kanaler[i].used == 1); i++)
        for(j = 0; (j < MAXCHAN && kanaler[i].program[j][0] != ' '); j++)
            if ((regexec(&rexp, kanaler[i].program[j], 0, NULL, 0)) == 0)  
                kanaler[i].watch[j] = 0;

    regfree(&rexp);
    

}

void flag_watch(const char *str) {

    int i,j;
    
    regex_t rexp;

    regcomp(&rexp, str, REG_EXTENDED | REG_ICASE);

    for(i = 0; (i < 16 && kanaler[i].used == 1); i++)
        for(j = 0; (j < MAXCHAN && kanaler[i].program[j][0] != ' '); j++)
            if ((regexec(&rexp, kanaler[i].program[j], 0, NULL, 0)) == 0)  
                kanaler[i].watch[j] = 1;

    regfree(&rexp);
    

}
void add_watch(const char *str) {

    FILE *f;
    
    int  currwatch = 0;
    char cfgfile[512];
    char buf[512];
    
    memset(&buf, 0, sizeof(buf));
    snprintf(cfgfile, sizeof(cfgfile), "%s/.tvconfig", getenv("HOME"));

    if ((f = fopen(cfgfile,"a")) == NULL) {
        beep();
        return;
    }

    snprintf(buf, sizeof(buf)-1, "WATCH %s\n",str);

    fputs(buf, f);
    
    fclose(f);

    for (currwatch=0; conf.watch[currwatch][0] != '\0'; currwatch++)
        ; 
    
    strcpy(conf.watch[currwatch], str);
    
    flag_watch(str); 
     
    return;
}

    
